'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'
' Purpose : TreeView drag & drop demo
'
' Author  : Paul Noble, http://www.zippety.net, demos@zippety.net
'
' Date    : 29 Aug 2002
'
' Remarks : TreeViewDrag & Drop demo
'           ------------------------

'1.  The purpose of this demo is to show you how to perform treeview drag &
'    drop using PowerBASIC.
'
'2.  This demo project builds upon techniques I read about in two programming
'    books -
'
'      a. Windows 95 Common Controls & Messages API Bible, by Richard Simon
'         Published by The Waite Group, ISBN 1-57169-101-7
'
'      b. Win32 Programming, by Brent E Rector and Joseph M Newcomer
'         Published by Addison-Wesley, ISBN 0-201-63492-9
'
'    Even though it was my firm belief that creating this demo did not infringe
'    copyright, I took the precaution of e-mailing both publishers, explaining
'    what I was doing, attaching the source code and requesting permission to
'    proceed.
'
'    The publisher for item (a), who is now Pearson Education, graciously gave
'    me permission to proceed. The publisher for item (b), despite having said
'    they would '...try to help solve my problem promptly', failed to actually
'    respond any further, despite being politely pursued by myself over a
'    period of several months. Therefore, I regret that I have had to remove
'    the code which shows how to automatically cause a treeview to scroll if 
'    you try to drag a tree item past the tree's vertical edges.
'
'3.  This code is surrendered into the Public Domain by the author
'
'4.  Any use of this code and sample program is entirely at the user's risk
'
'5.  SDK coding is used for maximum compatibility with PowerBASIC compiler 
'    versions. Written for PB/DLL 6.1, and later revised for PB/Win 7.0.
'
'6.  Programming note - notice that the tree does not flicker on resize (if you
'    have full window drag switched on in Windows)
'
'
' Hope this is useful to you -
'
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#compile  exe
#dim      all

'-----------------------------------------------------------------
' Equates:  73
'-----------------------------------------------------------------
%CS_VREDRAW                                     = &H1
%CS_HREDRAW                                     = &H2
%WINAPI                                         = 1
%TRUE                                           = 1
%FALSE                                          = 0
%NULL                                           = 0
%SB_VERT                                        = 1
%SIF_RANGE                                      = &H0001
%GWL_ID                                         = -12
%WM_CREATE                                      = &H1
%WM_DESTROY                                     = &H2
%WM_SIZE                                        = &H5
%WM_CLOSE                                       = &H10
%WM_NOTIFY                                      = &H4E
%WM_TIMER                                       = &H113
%WM_VSCROLL                                     = &H115
%WM_MOUSEMOVE                                   = &H200
%WM_LBUTTONUP                                   = &H202
%SIZE_RESTORED                                  = 0
%SIZE_MINIMIZED                                 = 1
%SIZE_MAXIMIZED                                 = 2
%WS_CHILD                                       = &H40000000
%WS_VISIBLE                                     = &H10000000
%WS_CLIPCHILDREN                                = &H02000000
%WS_CAPTION                                     = &H00C00000  ' WS_BORDER OR WS_DLGFRAME
%WS_VSCROLL                                     = &H00200000
%WS_SYSMENU                                     = &H00080000
%WS_THICKFRAME                                  = &H00040000
%WS_MINIMIZEBOX                                 = &H00020000
%WS_MAXIMIZEBOX                                 = &H00010000
%WS_EX_CLIENTEDGE                               = &H00000200
%HWND_DESKTOP                                   = 0
%SM_CYHSCROLL                                   = 3
%COLOR_BTNFACE                                  = 15
%IDC_ARROW                                      = 32512&
%ICC_TREEVIEW_CLASSES                           = &H00000002 ' treeview, tooltips
%TV_FIRST                                       = &H1100      ' TreeView messages
%TVN_FIRST                                      = 0-400       ' treeview
%ILC_MASK                                       = &H0001
%ILC_COLOR4                                     = &H0004
%ILC_COLOR16                                    = &H0010
%TVS_HASBUTTONS                                 = &H00000001
%TVS_HASLINES                                   = &H00000002
%TVS_SHOWSELALWAYS                              = &H00000020
%TVIF_TEXT                                      = &H0001
%TVIF_IMAGE                                     = &H0002
%TVIF_HANDLE                                    = &H0010
%TVIF_SELECTEDIMAGE                             = &H0020
%TVIS_EXPANDED                                  = &H0020
%TVI_ROOT                                       = &HFFFF0000???
%TVI_FIRST                                      = &HFFFF0001???
%TVI_LAST                                       = &HFFFF0002???
%TVM_INSERTITEM                                 = %TV_FIRST + 0
%TVM_DELETEITEM                                 = %TV_FIRST + 1
%TVM_EXPAND                                     = %TV_FIRST + 2
%TVE_COLLAPSE                                   = &H00000001
%TVE_EXPAND                                     = &H00000002
%TVM_GETITEMRECT                                = %TV_FIRST + 4
%TVM_GETINDENT                                  = %TV_FIRST + 6
%TVM_GETIMAGELIST                               = %TV_FIRST + 8
%TVSIL_NORMAL                                   = 0
%TVM_SETIMAGELIST                               = %TV_FIRST + 9
%TVM_GETNEXTITEM                                = %TV_FIRST + 10
%TVGN_ROOT                                      = &H0000
%TVGN_PARENT                                    = &H0003
%TVGN_CHILD                                     = &H0004
%TVGN_DROPHILITE                                = &H0008
%TVGN_CARET                                     = &H0009
%TVM_SELECTITEM                                 = %TV_FIRST + 11
%TVM_GETITEM                                    = %TV_FIRST + 12
%TVM_SETITEM                                    = %TV_FIRST + 13
%TVM_HITTEST                                    = %TV_FIRST + 17
%TVM_CREATEDRAGIMAGE                            = %TV_FIRST + 18
%TVN_ITEMEXPANDED                               = %TVN_FIRST - 6
%TVN_BEGINDRAG                                  = %TVN_FIRST - 7
%SM_CXSCREEN                                    = 0
%SM_CYSCREEN                                    = 1

'-----------------------------------------------------------------
' TYPE and UNION structures:  14
'-----------------------------------------------------------------
TYPE RECT
  nLeft AS LONG
  nTop AS LONG
  nRight AS LONG
  nBottom AS LONG
END TYPE

TYPE POINTAPI
  x AS LONG
  y AS LONG
END TYPE

TYPE tagMSG
  hwnd AS DWORD
  message AS DWORD
  wParam AS LONG
  lParam AS LONG
  time AS DWORD
  pt AS POINTAPI
END TYPE

TYPE WNDCLASSEX
    cbSize AS DWORD
    style AS DWORD
    lpfnWndProc AS LONG
    cbClsExtra AS LONG
    cbWndExtra AS LONG
    hInstance AS DWORD
    hIcon AS DWORD
    hCursor AS DWORD
    hbrBackground AS DWORD
    lpszMenuName AS ASCIIZ PTR
    lpszClassName AS ASCIIZ PTR
    hIconSm AS DWORD
END TYPE

TYPE ICONINFO
  fIcon AS LONG
  xHotspot AS DWORD
  yHotspot AS DWORD
  hbmMask AS LONG
  hbmColor AS LONG
END TYPE

TYPE NMHDR
  hwndFrom AS DWORD
  idfrom AS DWORD
  code AS LONG    ' used for messages, so needs to be LONG, not DWORD...
END TYPE

TYPE SCROLLINFO
  cbSize AS DWORD
  fMask AS DWORD
  nMin AS LONG
  nMax AS LONG
  nPage AS DWORD
  nPos AS LONG
  nTrackPos AS LONG
END TYPE

TYPE INIT_COMMON_CONTROLSEX
    dwSize AS DWORD   ' size of this structure
    dwICC  AS DWORD   ' flags indicating which classes to be initialized
END TYPE

TYPE TV_ITEM
    mask           AS DWORD
    hItem          AS DWORD
    STATE          AS DWORD
    stateMask      AS DWORD
    pszText        AS ASCIIZ PTR
    cchTextMax     AS LONG
    iImage         AS LONG
    iSelectedImage AS LONG
    cChildren      AS LONG
    lParam         AS LONG
END TYPE

TYPE TVITEMEX
    mask           AS DWORD
    hItem          AS DWORD
    STATE          AS DWORD
    stateMask      AS DWORD
    pszText        AS ASCIIZ PTR
    cchTextMax     AS LONG
    iImage         AS LONG
    iSelectedImage AS LONG
    cChildren      AS LONG
    lParam         AS LONG
    iIntegral      AS LONG
END TYPE

UNION TV_ITEM_UNION
    itemex AS TVITEMEX
    item   AS TV_ITEM
END UNION

TYPE TV_INSERTSTRUCT
    hParent      AS DWORD
    hInsertAfter AS DWORD
    item         AS TV_ITEM_UNION
END TYPE

TYPE TV_HITTESTINFO
    pt AS POINTAPI
    flags AS DWORD
    hItem AS DWORD
END TYPE

TYPE NM_TREEVIEW
    hdr     AS NMHDR
    action  AS DWORD
    itemOld AS TV_ITEM
    itemNew AS TV_ITEM
    ptDrag  AS POINTAPI
END TYPE

'-----------------------------------------------------------------
' Declared Functions:  42
'-----------------------------------------------------------------
DECLARE FUNCTION AdjustWindowRectEx LIB "USER32.DLL" ALIAS "AdjustWindowRectEx" (lpRect AS RECT, BYVAL dsStyle AS LONG, BYVAL bMenu AS LONG, BYVAL dwEsStyle AS DWORD) AS LONG
DECLARE FUNCTION SetRect LIB "USER32.DLL" ALIAS "SetRect" (lpRect AS RECT, BYVAL X1 AS LONG, BYVAL Y1 AS LONG, BYVAL X2 AS LONG, BYVAL Y2 AS LONG) AS LONG
DECLARE FUNCTION CreateWindowEx LIB "USER32.DLL" ALIAS "CreateWindowExA" (BYVAL dwExStyle AS DWORD, lpClassName AS ASCIIZ, lpWindowName AS ASCIIZ, BYVAL dwStyle AS DWORD, BYVAL x AS LONG, BYVAL y AS LONG, _
    BYVAL nWidth AS LONG, BYVAL nHeight AS LONG, BYVAL hWndParent AS DWORD, BYVAL hMenu AS DWORD, BYVAL hInstance AS DWORD, lpParam AS ANY) AS DWORD
DECLARE FUNCTION DefWindowProc LIB "USER32.DLL" ALIAS "DefWindowProcA" (BYVAL hWnd AS DWORD, BYVAL uMsg AS DWORD, BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG
DECLARE FUNCTION DeleteObject LIB "GDI32.DLL" ALIAS "DeleteObject" (BYVAL hObject AS DWORD) AS LONG
DECLARE FUNCTION DispatchMessage LIB "USER32.DLL" ALIAS "DispatchMessageA" (lpMsg AS tagMSG) AS LONG
DECLARE FUNCTION GetClientRect LIB "USER32.DLL" ALIAS "GetClientRect" (BYVAL hwnd AS DWORD, lpRect AS RECT) AS LONG
DECLARE FUNCTION GetCursorPos LIB "USER32.DLL" ALIAS "GetCursorPos" (lpPoint AS POINTAPI) AS LONG
DECLARE FUNCTION GetIconInfo LIB "USER32.DLL" ALIAS "GetIconInfo" (BYVAL hIcon AS DWORD, piconinfo AS ICONINFO) AS LONG
DECLARE FUNCTION GetMessage LIB "USER32.DLL" ALIAS "GetMessageA" (lpMsg AS tagMSG, BYVAL hWnd AS DWORD, BYVAL uMsgFilterMin AS DWORD, BYVAL uMsgFilterMax AS DWORD) AS LONG
DECLARE FUNCTION GetModuleHandle LIB "KERNEL32.DLL" ALIAS "GetModuleHandleA" (lpModuleName AS ASCIIZ) AS DWORD
DECLARE FUNCTION GetScrollInfo LIB "USER32.DLL" ALIAS "GetScrollInfo" (BYVAL hWnd AS DWORD, BYVAL n AS LONG, lpScrollInfo AS SCROLLINFO) AS LONG
DECLARE FUNCTION GetSystemMetrics LIB "USER32.DLL" ALIAS "GetSystemMetrics" (BYVAL nIndex AS LONG) AS LONG
DECLARE FUNCTION ImageList_AddMasked LIB "COMCTL32.DLL" _
    ALIAS "ImageList_AddMasked" (BYVAL himl AS DWORD, _
    BYVAL hbmImage AS DWORD, BYVAL crMask AS DWORD) AS LONG
DECLARE FUNCTION ImageList_BeginDrag LIB "COMCTL32.DLL" _
    ALIAS "ImageList_BeginDrag" (BYVAL himlTrack AS DWORD, _
    BYVAL iTrack AS LONG, BYVAL dxHotSpot AS LONG, BYVAL dyHotSpot AS LONG) _
    AS LONG
DECLARE FUNCTION ImageList_Create LIB "COMCTL32.DLL" ALIAS "ImageList_Create" _
    (BYVAL cx AS LONG, BYVAL cy AS LONG, BYVAL flags AS DWORD, _
    BYVAL cInitial AS LONG, BYVAL cGrow AS LONG) AS DWORD
DECLARE FUNCTION ImageList_Destroy LIB "COMCTL32.DLL" _
    ALIAS "ImageList_Destroy" (BYVAL himl AS DWORD) AS LONG
DECLARE FUNCTION ImageList_DragEnter LIB "COMCTL32.DLL" _
    ALIAS "ImageList_DragEnter" (BYVAL hWndLock AS DWORD, BYVAL x AS LONG, _
    BYVAL y AS LONG) AS LONG
DECLARE FUNCTION ImageList_DragLeave LIB "COMCTL32.DLL" _
    ALIAS "ImageList_DragLeave" (BYVAL hWndLock AS DWORD) AS LONG
DECLARE FUNCTION ImageList_DragMove LIB "COMCTL32.DLL" _
    ALIAS "ImageList_DragMove" (BYVAL x AS LONG, BYVAL y AS LONG) AS LONG
DECLARE FUNCTION ImageList_DragShowNolock LIB "COMCTL32.DLL" _
    ALIAS "ImageList_DragShowNolock" (BYVAL fShow AS LONG) AS LONG
DECLARE FUNCTION ImageList_ReplaceIcon LIB "COMCTL32.DLL" _
    ALIAS "ImageList_ReplaceIcon" (BYVAL himl AS DWORD, BYVAL i AS LONG, _
    BYVAL hIcon AS DWORD) AS LONG
DECLARE FUNCTION ImageList_SetDragCursorImage LIB "COMCTL32.DLL" _
    ALIAS "ImageList_SetDragCursorImage" (BYVAL himlDrag AS DWORD, _
    BYVAL iDrag AS LONG, BYVAL dxHotSpot AS LONG, BYVAL dyHotSpot AS LONG) _
    AS LONG
DECLARE FUNCTION InitCommonControlsEx LIB "COMCTL32.DLL" ALIAS "InitCommonControlsEx" (icc AS INIT_COMMON_CONTROLSEX) AS LONG
DECLARE FUNCTION InvalidateRect LIB "USER32.DLL" ALIAS "InvalidateRect" (BYVAL hWnd AS DWORD, lpRect AS RECT, BYVAL bErase AS LONG) AS LONG
DECLARE FUNCTION KillTimer LIB "USER32.DLL" ALIAS "KillTimer" (BYVAL hWnd AS DWORD, BYVAL nIDEvent AS LONG) AS LONG
DECLARE FUNCTION LoadBitmap LIB "USER32.DLL" ALIAS "LoadBitmapA" (BYVAL hInstance AS DWORD, lpBitmapName AS ASCIIZ) AS DWORD
DECLARE FUNCTION LoadCursor LIB "USER32.DLL" ALIAS "LoadCursorA" (BYVAL hInstance AS DWORD, lpCursorName AS ASCIIZ) AS DWORD
DECLARE FUNCTION MapWindowPoints LIB "USER32.DLL" ALIAS "MapWindowPoints" (BYVAL hWndFrom AS DWORD, BYVAL hWndTo AS DWORD, lppt AS ANY, BYVAL cPoints AS LONG) AS LONG
DECLARE FUNCTION MoveWindow LIB "USER32.DLL" ALIAS "MoveWindow" (BYVAL hWnd AS DWORD, BYVAL x AS LONG, BYVAL y AS LONG, BYVAL nWidth AS LONG, BYVAL nHeight AS LONG, BYVAL bRepaint AS LONG) AS LONG
DECLARE FUNCTION PtInRect LIB "USER32.DLL" ALIAS "PtInRect" (lpRect AS RECT, BYVAL ptx AS LONG, BYVAL pty AS LONG) AS LONG
DECLARE FUNCTION RegisterClassEx LIB "USER32.DLL" ALIAS "RegisterClassExA" (pcWndClassEx AS WNDCLASSEX) AS WORD
DECLARE FUNCTION ReleaseCapture LIB "USER32.DLL" ALIAS "ReleaseCapture" () AS LONG
DECLARE FUNCTION ScreenToClient LIB "USER32.DLL" ALIAS "ScreenToClient" (BYVAL hWnd AS DWORD, lpPoint AS POINTAPI) AS LONG
DECLARE FUNCTION SendMessage LIB "USER32.DLL" ALIAS "SendMessageA" (BYVAL hWnd AS DWORD, BYVAL dwMsg AS DWORD, BYVAL wParam AS DWORD, BYVAL lParam AS LONG) AS LONG
DECLARE FUNCTION SetCapture LIB "USER32.DLL" ALIAS "SetCapture" (BYVAL hWnd AS DWORD) AS LONG
DECLARE FUNCTION SetTimer LIB "USER32.DLL" ALIAS "SetTimer" (BYVAL hWnd AS DWORD, BYVAL nIDEvent AS LONG, BYVAL uElapse AS DWORD, BYVAL lpTimerFunc AS LONG) AS LONG
DECLARE FUNCTION SetWindowLong LIB "USER32.DLL" ALIAS "SetWindowLongA" (BYVAL hWnd AS DWORD, BYVAL nIndex AS LONG, BYVAL lNewLong AS LONG) AS LONG
DECLARE FUNCTION ShowCursor LIB "USER32.DLL" ALIAS "ShowCursor" (BYVAL bShow AS LONG) AS LONG
DECLARE FUNCTION ShowWindow LIB "USER32.DLL" ALIAS "ShowWindow" (BYVAL hWnd AS DWORD, BYVAL nCmdShow AS LONG) AS LONG
DECLARE FUNCTION TranslateMessage LIB "USER32.DLL" ALIAS "TranslateMessage" (lpMsg AS tagMSG) AS LONG
DECLARE FUNCTION UpdateWindow LIB "USER32.DLL" ALIAS "UpdateWindow" (BYVAL hWnd AS DWORD) AS LONG

'-----------------------------------------------------------------
' Declared Subs:  3
'-----------------------------------------------------------------
DECLARE SUB ImageList_EndDrag LIB "COMCTL32.DLL" ALIAS "ImageList_EndDrag" ()
DECLARE SUB InitCommonControls LIB "COMCTL32.DLL" ALIAS "InitCommonControls" ()
DECLARE SUB PostQuitMessage LIB "USER32.DLL" ALIAS "PostQuitMessage" (BYVAL nExitCode AS LONG)

'-----------------------------------------------------------------
' Functions:  20  (begins with declarations)
'-----------------------------------------------------------------
DECLARE FUNCTION CreateWindow (lpClassName AS ASCIIZ, lpWindowName AS ASCIIZ, BYVAL dwStyle AS DWORD, BYVAL xx AS LONG, BYVAL yy AS LONG, BYVAL nWidth AS LONG, _
                                 BYVAL nHeight AS LONG, BYVAL hwndParent AS DWORD, BYVAL hMenu AS DWORD, BYVAL hInstance AS DWORD, BYVAL lpParam AS DWORD) AS LONG
DECLARE FUNCTION ImageList_AddIcon (BYVAL hIml AS DWORD, hIcon AS DWORD) AS LONG
DECLARE FUNCTION TreeView_InsertItem (BYVAL hWnd AS LONG, lpis AS TV_INSERTSTRUCT) _
           AS DWORD
DECLARE FUNCTION TreeView_DeleteItem (BYVAL hWnd AS DWORD, BYVAL hitem AS DWORD) _
           AS LONG
DECLARE FUNCTION TreeView_Expand (BYVAL hWnd AS DWORD, BYVAL hitem AS LONG, _
           BYVAL code AS DWORD) AS LONG
DECLARE FUNCTION TreeView_GetItemRect (BYVAL hWnd AS DWORD, BYVAL hItem AS DWORD, _
           prc AS RECT, BYVAL code AS LONG) AS LONG
DECLARE FUNCTION TreeView_GetIndent (BYVAL hWnd AS DWORD) AS DWORD
DECLARE FUNCTION TreeView_GetImageList (BYVAL hWnd AS DWORD, BYVAL iImage AS LONG) _
           AS DWORD
DECLARE FUNCTION TreeView_SetImageList (BYVAL hWnd AS DWORD, BYVAL himl AS DWORD, _
           BYVAL iImage AS LONG) AS DWORD
DECLARE FUNCTION TreeView_GetNextItem (BYVAL hWnd AS DWORD, BYVAL hitem AS DWORD, _
           BYVAL code AS DWORD) AS DWORD
DECLARE FUNCTION TreeView_GetParent (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
           AS DWORD
DECLARE FUNCTION TreeView_GetDropHilight (BYVAL hwnd AS DWORD) AS DWORD
DECLARE FUNCTION TreeView_GetRoot (BYVAL hwnd AS DWORD) AS DWORD
DECLARE FUNCTION TreeView_Select (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD, _
           BYVAL code AS LONG) AS LONG
DECLARE FUNCTION TreeView_SelectItem (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
           AS LONG
DECLARE FUNCTION TreeView_SelectDropTarget (BYVAL hwnd AS DWORD, _
           BYVAL hitem AS DWORD) AS LONG
DECLARE FUNCTION TreeView_GetItem (BYVAL hwnd AS DWORD, pitem AS TV_ITEM) AS LONG
DECLARE FUNCTION TreeView_SetItem (BYVAL hwnd AS DWORD, pitem AS TV_ITEM) AS LONG
DECLARE FUNCTION TreeView_HitTest (BYVAL hwnd AS DWORD, lpht AS TV_HITTESTINFO) _
           AS DWORD
DECLARE FUNCTION TreeView_CreateDragImage (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
           AS DWORD
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
FUNCTION CreateWindow (lpClassName AS ASCIIZ, lpWindowName AS ASCIIZ, BYVAL dwStyle AS DWORD, BYVAL xx AS LONG, BYVAL yy AS LONG, BYVAL nWidth AS LONG, _
                         BYVAL nHeight AS LONG, BYVAL hwndParent AS DWORD, BYVAL hMenu AS DWORD, BYVAL hInstance AS DWORD, BYVAL lpParam AS DWORD) AS LONG
    FUNCTION = CreateWindowEx(0, lpClassName, lpWindowName, dwStyle, xx, yy, nWidth, nHeight, hWndParent, hMenu, hInstance, BYVAL lpParam)
END FUNCTION

FUNCTION ImageList_AddIcon (BYVAL hIml AS DWORD, hIcon AS DWORD) AS LONG
    FUNCTION = ImageList_ReplaceIcon(hIml, -1, hIcon)
END FUNCTION

FUNCTION TreeView_InsertItem (BYVAL hWnd AS LONG, lpis AS TV_INSERTSTRUCT) _
   AS DWORD
    FUNCTION = SendMessage(hWnd, %TVM_INSERTITEM, 0, VARPTR(lpis))
END FUNCTION

FUNCTION TreeView_DeleteItem (BYVAL hWnd AS DWORD, BYVAL hitem AS DWORD) _
   AS LONG
    FUNCTION = SendMessage(hWnd, %TVM_DELETEITEM, 0, hitem)
END FUNCTION

FUNCTION TreeView_Expand (BYVAL hWnd AS DWORD, BYVAL hitem AS LONG, _
   BYVAL code AS DWORD) AS LONG
    FUNCTION = SendMessage(hWnd, %TVM_EXPAND, code, hitem)
END FUNCTION

FUNCTION TreeView_GetItemRect (BYVAL hWnd AS DWORD, BYVAL hItem AS DWORD, _
   prc AS RECT, BYVAL code AS LONG) AS LONG
    prc.nLeft = hItem
    FUNCTION = SendMessage(hWnd, %TVM_GETITEMRECT, code, VARPTR(prc))
END FUNCTION

FUNCTION TreeView_GetIndent (BYVAL hWnd AS DWORD) AS DWORD
    FUNCTION = SendMessage(hWnd, %TVM_GETINDENT, 0, 0)
END FUNCTION

FUNCTION TreeView_GetImageList (BYVAL hWnd AS DWORD, BYVAL iImage AS LONG) _
   AS DWORD
    FUNCTION = SendMessage(hWnd, %TVM_GETIMAGELIST, iImage, 0)
END FUNCTION

FUNCTION TreeView_SetImageList (BYVAL hWnd AS DWORD, BYVAL himl AS DWORD, _
   BYVAL iImage AS LONG) AS DWORD
    FUNCTION = SendMessage(hWnd, %TVM_SETIMAGELIST, iImage, himl)
END FUNCTION

FUNCTION TreeView_GetNextItem (BYVAL hWnd AS DWORD, BYVAL hitem AS DWORD, _
   BYVAL code AS DWORD) AS DWORD
    FUNCTION = SendMessage(hWnd, %TVM_GETNEXTITEM, code, hitem)
END FUNCTION

FUNCTION TreeView_GetParent (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
   AS DWORD
    FUNCTION = TreeView_GetNextItem(hwnd, hitem, %TVGN_PARENT)
END FUNCTION

FUNCTION TreeView_GetDropHilight (BYVAL hwnd AS DWORD) AS DWORD
    FUNCTION = TreeView_GetNextItem(hwnd, %NULL, %TVGN_DROPHILITE)
END FUNCTION

FUNCTION TreeView_GetRoot (BYVAL hwnd AS DWORD) AS DWORD
    FUNCTION = TreeView_GetNextItem(hwnd, %NULL, %TVGN_ROOT)
END FUNCTION

FUNCTION TreeView_Select (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD, _
   BYVAL code AS LONG) AS LONG
    FUNCTION = SendMessage(hwnd, %TVM_SELECTITEM, code, hitem)
END FUNCTION

FUNCTION TreeView_SelectItem (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
   AS LONG
    FUNCTION = TreeView_Select(hwnd, hitem, %TVGN_CARET)
END FUNCTION

FUNCTION TreeView_SelectDropTarget (BYVAL hwnd AS DWORD, _
   BYVAL hitem AS DWORD) AS LONG
    FUNCTION = TreeView_Select(hwnd, hitem, %TVGN_DROPHILITE)
END FUNCTION

FUNCTION TreeView_GetItem (BYVAL hwnd AS DWORD, pitem AS TV_ITEM) AS LONG
    FUNCTION = SendMessage(hwnd, %TVM_GETITEM, 0, VARPTR(pitem))
END FUNCTION

FUNCTION TreeView_SetItem (BYVAL hwnd AS DWORD, pitem AS TV_ITEM) AS LONG
    FUNCTION = SendMessage(hwnd, %TVM_SETITEM, 0, VARPTR(pitem))
END FUNCTION

FUNCTION TreeView_HitTest (BYVAL hwnd AS DWORD, lpht AS TV_HITTESTINFO) _
   AS DWORD
    FUNCTION = SendMessage(hwnd, %TVM_HITTEST, 0, VARPTR(lpht))
END FUNCTION

FUNCTION TreeView_CreateDragImage (BYVAL hwnd AS DWORD, BYVAL hitem AS DWORD) _
   AS DWORD
    FUNCTION = SendMessage(hwnd, %TVM_CREATEDRAGIMAGE, 0, hitem)
END FUNCTION


#Include "WINLIFT.INC"

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'
'
' Main include file for TreeView drag & drop demo
'
'
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
$TITLE           = "Drag & drop TreeView"
$CLASS           = "TDD_Demo"

'Control IDs...
%ID_TREE         = 500

'Tree bitmaps...
%IDB_BOOK_C      = 100
%IDB_BOOK_O      = 101
%IDB_FOLDER_C    = 102
%IDB_FOLDER_O    = 103
%IDB_DOCUMENT    = 104

'Tree ordinals...
%IDO_BOOK_C      = 0
%IDO_BOOK_O      = 1
%IDO_FOLDER_C    = 2
%IDO_FOLDER_O    = 3
%IDO_DOCUMENT    = 4

'Max length of treeview text, including terminating null
%TV_ITEM_CHARS   = 100

'Tree scrolling...
%SCROLL_UP       = 0
%SCROLL_DOWN     = 1
%SCROLL_OFF      = 0
%SCROLL_SLOWLY   = 1
%SCROLL_QUICKLY  = 2

'Tree scroll timer...
%IDT_SCROLL      = 200

%XMIN_SIZE       = 320
%YMIN_SIZE       = 292

#resource "dragdrop.pbr"

'For convenience, we use 2 globals to keep track of treeview automatic scrolling
global gyScrollTimerActive as long
global gmScrollDirection   as long

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub TVW_SetImage( byval hTree as dword, byval hItem as dword, byval yExpanded as long )

  local uItem          as TV_ITEM
  local mImage         as long
  local mNewImage      as long

  'Find out what image is used by this item...
  uItem.hItem    = hItem                                                  'this is the item handle we want it for
  uItem.mask     = %TVIF_HANDLE or %TVIF_IMAGE or %TVIF_SELECTEDIMAGE     'vital to specify both images
  TreeView_GetItem hTree, uItem                                           'fetch it
  mImage         = uItem.iImage
  mNewImage      = -1                                                     'impossible value

  'Decide what (if anything) we need to change the image to...
  select case mImage

    case %IDO_BOOK_C
      if yExpanded <> 0 then 
        if TreeView_GetNextItem( hTree, hItem, %TVGN_CHILD ) <> %NULL then
          mNewImage = %IDO_BOOK_O
        end if
      end if
    
    case %IDO_BOOK_O
      if yExpanded =  0 then mNewImage = %IDO_BOOK_C
    
    case %IDO_FOLDER_C
      if yExpanded <> 0 then 
        if TreeView_GetNextItem( hTree, hItem, %TVGN_CHILD ) <> %NULL then
          mNewImage = %IDO_FOLDER_O
        end if
      end if
    
    case %IDO_FOLDER_O
      if yExpanded =  0 then mNewImage = %IDO_FOLDER_C

  end select

  'Is a change required ?
  if mNewImage <> -1 then

    uItem.iImage = mNewImage
    uItem.iSelectedImage = mNewImage
    TreeView_SetItem hTree, uItem

  end if

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub TVW_Expand( byval hTree as dword, byval hItem as dword, byval yExpand as long )

  'Purpose : Wrapper for the TreeView_Expand() macro, but adds the automatic adjustment
  '          of the image according to the new collapsed/expanded state
  '
  'Remarks : It turns out that if the *currently selected* item is expanded by code, no adjustment
  '          to its 'expanded' image takes place, so we do that here, if necessary
  '
  local uItem          as TV_ITEM
  local mImage         as long
  local mToggledImage  as long

  'Perform the basic action...
  if yExpand <> 0 then
    TreeView_Expand hTree, hItem, %TVE_EXPAND
  else
    TreeView_Expand hTree, hItem, %TVE_COLLAPSE
  end if

  'Update the item's images...
  TVW_SetImage hTree, hItem, yExpand

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function CreateTree( byval hInstance as dword, byval hParent as dword, rc as RECT ) as dword

  'Purpose : Creates the treeview control and sets up the image list
  '
  'Returns : Treeview handle
  '
  'Remarks : Does not create any items in the tree
  '
  local hBitmap  as dword
  local hRoot    as dword
  local hPItem   as dword
  local hTree    as dword
  local hImages  as dword
  local nStyle, mMaskRGB as long

  'Create the treeview...
  nStyle = %WS_CHILD or %WS_VISIBLE or %WS_VSCROLL or %TVS_HASLINES or %TVS_HASBUTTONS or %TVS_SHOWSELALWAYS
  hTree = CreateWindowEx(%WS_EX_CLIENTEDGE, "SysTreeView32", "", nStyle, _
                          rc.nLeft + 20, rc.nTop + 20, rc.nRight - rc.nLeft - 40, rc.nBottom - rc.nTop - 40, _
                          hParent, %NULL, hInstance, byval %NULL)

  if hTree then

    hImages = ImageList_Create( 16, 16, %ILC_COLOR16 or %ILC_MASK, 6, 0 )
    TreeView_SetImageList hTree, hImages, %TVSIL_NORMAL

    'This is the mask colour we always use in our images...
    mMaskRGB = RGB(255,0,255)

    'Add bitmaps from the resource file to the image list
    
    'Item 0 (%IDO_BOOK_C)...
    hBitmap = LoadBitmap(hInstance, byval %IDB_BOOK_C)
    ImageList_AddMasked hImages, hBitmap, mMaskRGB
    DeleteObject hBitmap
    
    'Item 1 (%IDO_BOOK_O)...
    hBitmap = LoadBitmap(hInstance, byval %IDB_BOOK_O)
    ImageList_AddMasked hImages, hBitmap, mMaskRGB
    DeleteObject hBitmap
    
    'Item 2 (%IDO_FOLDER_C)...
    hBitmap = LoadBitmap(hInstance, byval %IDB_FOLDER_C)
    ImageList_AddMasked hImages, hBitmap, mMaskRGB
    DeleteObject hBitmap
    
    'Item 3 (%IDO_FOLDER_O)...
    hBitmap = LoadBitmap(hInstance, byval %IDB_FOLDER_O)
    ImageList_AddMasked hImages, hBitmap, mMaskRGB
    DeleteObject hBitmap
    
    'Item 4 (%IDO_DOCUMENT)...
    hBitmap = LoadBitmap(hInstance, byval %IDB_DOCUMENT)
    ImageList_AddMasked hImages, hBitmap, mMaskRGB
    DeleteObject hBitmap
  
  end if
  
  'We're done
  function = hTree

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub PopulateTree( byval hTree as dword )

  'Purpose : Adds dummy data to the tree
  '
  dim   sLName()   as string
  dim   sFName()   as string
  
  local uInsert    as TV_INSERTSTRUCT
  local hRoot      as dword
  local hFolder    as dword
  local hFile      as dword

  local zText      as asciiz * %TV_ITEM_CHARS
  local mFolder    as long
  local mFile      as long

  'Set up the insertion structure...
  uInsert.hInsertAfter             = %TVI_LAST
  uInsert.item.item.mask           = %TVIF_TEXT or %TVIF_IMAGE or %TVIF_SELECTEDIMAGE

  'Root node...
  zText                            = "Family Visits"
  uInsert.hParent                  = %TVI_ROOT
  uInsert.item.item.pszText        = varptr( zText )
  uInsert.item.item.iImage         = %IDO_BOOK_C
  uInsert.item.item.iSelectedImage = %IDO_BOOK_C
  uInsert.item.item.LPARAM         = %NULL
  hRoot                            = TreeView_InsertItem( hTree, uInsert )
  
  'Create some dummy data...
  redim sLName( 1 to 5 )
  sLName(1) = "Smith"
  sLName(2) = "Jones"
  sLName(3) = "Clark"
  sLName(4) = "Green"
  sLName(5) = "Taylor"
  
  redim sFName( 1 to 5 )
  sFName(1) = "John"
  sFName(2) = "Susan"
  sFName(3) = "Bridget"
  sFName(4) = "Alan"
  sFName(5) = "Carol"
  
  'Add some folders and files...
  for mFolder = 1 to ubound( sLName )
  
    'Add folder...
    zText                            = sLName(mFolder) & " family"
    uInsert.hParent                  = hRoot
    uInsert.item.item.pszText        = varptr( zText )
    uInsert.item.item.iImage         = %IDO_FOLDER_C
    uInsert.item.item.iSelectedImage = %IDO_FOLDER_C
    uInsert.item.item.LPARAM         = %NULL
    hFolder                          = TreeView_InsertItem( hTree, uInsert )
    
    'Add files to this folder...
    for mFile = 1 to ubound( sFName )
      
      zText                            = sFName(mFile) & " " & sLName(mFolder)
      uInsert.hParent                  = hFolder
      uInsert.item.item.pszText        = varptr( zText )
      uInsert.item.item.iImage         = %IDO_DOCUMENT
      uInsert.item.item.iSelectedImage = %IDO_DOCUMENT
      uInsert.item.item.LPARAM         = %NULL
      hFile                            = TreeView_InsertItem( hTree, uInsert )
      
    next mFile
    
    'Expand say the first 2 folders...
    if mFolder <= 2 then TVW_Expand hTree, hFolder, %TRUE
  
  next mFolder
  
  'Expand the root node, so that all the first-level nodes are visible...
  TVW_Expand hTree, hRoot, %TRUE

Bye_BuildTree:
  erase sLName
  erase sFName
  exit sub

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_GetImage( byval hTree as dword, byval hItem as dword ) as long

  'Purpose : Returns the ordinal of the item's bitmap ; if the item is not
  '          found, returns -1
  '
  local ti as TV_ITEM

  ti.hItem = hItem                                              'set up the item structure
  ti.mask  = %TVIF_HANDLE or %TVIF_IMAGE                        'these are the valid fields

  if TreeView_GetItem( hTree, ti ) = 0 then                     'fetch info about the new item
    function = -1
  else
    function = ti.iImage
  end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_GetLevel( byval hTree as dword, byval hItem as dword ) as long

  'Purpose : Returns the item's level. The root item is assumed to be at
  '          level = zero.
  '
  local hRoot  as dword
  local mLevel as long
    
  if hItem <> %NULL then
  
    'Determine the root's handle...
    hRoot = TreeView_GetRoot( hTree )
    
    'Walk back up the tree, towards the root...
    while hItem <> hRoot

      incr mLevel
      hItem = TreeView_GetNextItem( hTree, hItem, %TVGN_PARENT )

    wend
  
  end if
  
  function = mLevel

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_IsDragSource( byval hTree as dword, byval hItem as dword ) as long

  'Purpose : Determines whether the specified tree item is allowed to be dragged
  '
  'Returns : 1 ... yes
  '          0 ... no
  '
  'Remarks : In this demo, we allow only 'people' to be dragged
  '
  select case TVW_GetImage( hTree, hItem )
  
    case %IDO_DOCUMENT
      function = 1
      
    case else
      function = 0
  
  end select

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_IsDropTarget( byval hTree as dword, byval hItemDest as dword ) as long
  
  'Purpose : Determines whether <hItemDest> is a valid drop target
  '
  'Returns : 0 or 1
  '
  'Remarks : The rules -
  '
  '            1. We can drop people onto a family or another person
  '            2. We cannot drop people onto the root
  '
  select case TVW_GetImage( hTree, hItemDest )        
    
    case %IDO_FOLDER_C, %IDO_FOLDER_O, %IDO_DOCUMENT
      function = 1
    
    case else
      function = 0
  
  end select
      
end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_GetText( byval hTree as dword, byval hItem as dword ) as string

  'Purpose : Returns the item's text ; if the item is not found, returns ""
  '
  local ti      as TV_ITEM
  local zText   as asciiz * ( %TV_ITEM_CHARS + 1 )
  local s       as string  

  'Set up the item structure...
  ti.hItem      = hItem                                        
  ti.mask       = %TVIF_TEXT                                    
  zText         = space$( %TV_ITEM_CHARS )                      
  ti.cchTextMax = %TV_ITEM_CHARS + 1                                
  ti.pszText    = varptr( zText )                              
  
  'Query the tree...
  if TreeView_GetItem( hTree, ti ) = 0 then                     'call failed                  
    function = ""
  else
    s = zText
    function = s
  end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_IsChildOf( byval hTree as dword, byval hItem as dword, byval hPossParent as dword ) as long

  'Purpose : Determines whether <hItem> is a child of <hPossParent>
  '
  'Returns : 0 or 1
  '
  local mResult as long
  local hRoot   as dword
  
  if hItem <> %NULL and hPossParent <> %NULL then
  
    'Determine the root's handle...
    hRoot = TreeView_GetRoot( hTree )
    
    'Walk back up the tree, towards the root...
    while hItem <> hRoot

      if hItem = hPossParent then
        mResult = 1
        exit loop
      else
        hItem = TreeView_GetNextItem( hTree, hItem, %TVGN_PARENT )
      end if

    wend
  
  end if
  
  function = mResult

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_HasChildren( byval hTree as dword, byval hItem as dword ) as long

  'Purpose : Determines whether the specified item has any children under it
  '          item
  '
  'Returns : 1 ... it does
  '          0 ... it doesn't
  '
  if TreeView_GetNextItem( hTree, hItem, %TVGN_CHILD ) = %NULL then
    function = 0
  else
    function = 1
  end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function TVW_MoveItem( byval hTree as dword, byval hItemSource as dword, byval hItemDest as dword ) as dword

  'Purpose : Move a tree item following a successful drag & drop operation
  '
  'Returns : Handle of the newly-created item
  '
  'Remarks : In this demo, we only copy across the dragged item's image and text, but
  '          in general, you may want to also take across other attributes such as
  '          the item's LPARAM, expansion state, text bolding, etc
  '
  local uInsert       as TV_INSERTSTRUCT
  
  local hSourceParent as dword
  local hDestParent   as dword
  local zText         as asciiz * %TV_ITEM_CHARS
  
  local hNewItem      as dword
  

  'Start off the insertion structure...
  uInsert.item.item.mask           = %TVIF_TEXT or %TVIF_IMAGE or %TVIF_SELECTEDIMAGE
  zText                            = TVW_GetText( hTree, hItemSource )
  uInsert.item.item.pszText        = varptr( zText ) 
  uInsert.item.item.iImage         = TVW_GetImage ( hTree, hItemSource )
  uInsert.item.item.iSelectedImage = uInsert.item.item.iImage
  
  'Are we dropping onto a folder, or onto another person ?
  select case TVW_GetImage( hTree, hItemDest )        

    case %IDO_DOCUMENT

      'Insert the dragged person after the dropped-on person
      uInsert.hParent                  = TreeView_GetParent( hTree, hItemDest )
      uInsert.hInsertAfter             = hItemDest

    case %IDO_FOLDER_C, %IDO_FOLDER_O

      'Insert the dragged person as the first person under this folder
      uInsert.hParent                  = hItemDest
      uInsert.hInsertAfter             = %TVI_FIRST

    case else          
      beep : goto Bye_TVW_MoveItem

  end select

  'Create the new item...
  hNewItem = TreeView_InsertItem( hTree, uInsert )

  'Did we succeed ?
  if hNewItem then
    
    'Record these...
    hSourceParent = TreeView_GetParent( hTree, hItemSource )
    hDestParent   = TreeView_GetParent( hTree, hItemDest   )
    
    'Now it's safe to delete the source item...
    TreeView_DeleteItem hTree, hItemSource

    'Ensure the new item is visible and selected; we don't use TreeView_EnsureVisible(), 
    'because that can leave an expanded parent's image in the wrong state...  
    TVW_Expand hTree, hDestParent, 1
    TreeView_SelectItem hTree, hNewItem

    'If the source folder has been emptied, close it up...          
    if TVW_HasChildren( hTree, hSourceParent ) = 0 then      
      TVW_Expand hTree, hSourceParent, 0
    end if
  
  end if

Bye_TVW_MoveItem:
  function = hNewItem
  
end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub AutomaticScroll( byval hDialog as dword, byval hTree as dword, byval mWay as long, byval mSpeed as long )

  'Purpose : Automatic scrolling of <hTree> during drag operations. Called by
  '          CheckAutomaticScroll(), and by main message loop
  '
  select case mSpeed
  
    case %SCROLL_OFF
      if gyScrollTimerActive then
        KillTimer hDialog, %IDT_SCROLL
        gyScrollTimerActive = 0
      end if
    
    case %SCROLL_SLOWLY
      gyScrollTimerActive = 1
      SetTimer hDialog, %IDT_SCROLL, 200, %NULL
    
    case %SCROLL_QUICKLY
      gyScrollTimerActive = 1
      SetTimer hDialog, %IDT_SCROLL, 50,  %NULL
  
  end select
  
  gmScrollDirection = mWay

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

sub CheckForAutomaticScroll( byval hDialog as dword, byval hTree as dword, uPoint as POINTAPI )

  'Purpose : Checks to see whether, based on <uPoint>, we need to
  '          auto-scroll the treeview.
  '
  'Remarks : <uPoint> is in tree co-ordinates, since the tree has
  '          captured the mouse
  '
  local si          as SCROLLINFO
  local uRect       as RECT
  local mZoneHeight as long
  
  si.fMask  = %SIF_RANGE                                        'requested data
  si.cbSize = sizeof(si)
  
  if GetScrollInfo( hTree, %SB_VERT, si ) then                  'if data was found  
    if si.nMin <> si.nMax then                                  'has a scroll bar
      
      'Within this far of the top/bottom of the tree,
      'we will auto-scroll...
      mZoneHeight = GetSystemMetrics( %SM_CYHSCROLL )
      
      'Are we inside the tree, or outside ?
      GetClientRect hTree, uRect
      if PtInRect( uRect, uPoint.x, uPoint.y ) then             'inside the tree
      
        if uPoint.y < uRect.nTop + mZoneHeight then
          AutomaticScroll hDialog, hTree, %SCROLL_UP, %SCROLL_SLOWLY         
        else
          if uPoint.y > uRect.nBottom - mZoneHeight then
            AutomaticScroll hDialog, hTree, %SCROLL_DOWN, %SCROLL_SLOWLY
          else
            AutomaticScroll hDialog, hTree, %SCROLL_DOWN, %SCROLL_OFF
          end if
        end if
      
      else                                                      'outside the tree
      
        if uPoint.y < uRect.nTop then
          AutomaticScroll hDialog, hTree, %SCROLL_UP, %SCROLL_QUICKLY
        else
          if uPoint.y > uRect.nBottom then
            AutomaticScroll hDialog, hTree, %SCROLL_DOWN, %SCROLL_QUICKLY
          end if
        end if
      
      end if
      
    end if
  end if

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function WinMain ( byval hInstance     as long, _
                   byval hPrevInstance as long, _
                   BYVAL lpCmdLine     as asciiz ptr, _
                   byval iCmdShow      as long ) as long

  'This is all standard SDK stuff ; create a dialog and display it
  
  local uCC     as INIT_COMMON_CONTROLSEX
  local uClass  as WndClassEx
  local msg     as tagMsg
  
  local zTitle  as asciiz * 30
  local zClass  as asciiz * 30
  local hDialog as dword
  local rc      as RECT
  local x, y, dwExStyle, dwStyle as LONG

  'Initialise...
  zTitle               = $TITLE
  zClass               = $CLASS
  
  'Register main window class...
  uClass.cbSize        = sizeof(uClass)
  uClass.style         = %CS_HREDRAW OR %CS_VREDRAW
  uClass.lpfnwndproc   = codeptr(wndproc)
  uClass.cbClsExtra    = 0
  uClass.cbWndExtra    = 0
  uClass.hInstance     = hInstance
  uClass.hIcon         = %NULL
  uClass.hCursor       = %NULL
  uClass.hbrBackground = %NULL'%COLOR_BTNFACE + 1
  uClass.lpszMenuName  = %NULL
  uClass.lpszCLASSNAME = varptr(zClass)
  RegisterClassEx uClass

  'Initialise the common controls module...
  uCC.dwSize           = sizeof(uCC)
  uCC.dwICC            = %ICC_TREEVIEW_CLASSES
  if InitCommonControlsEx(uCC) = 0 then
    InitCommonControls                                          'Win95A
  end if
  
  dwExStyle = 0
  dwStyle = %WS_CAPTION or %WS_SYSMENU or %WS_THICKFRAME or %WS_MINIMIZEBOX or %WS_MAXIMIZEBOX 
  CALL SetRect(rc, 0, 0, %XMIN_SIZE, %YMIN_SIZE)
  CALL AdjustWindowRectEx(rc, dwStyle, %FALSE, dwExStyle)  ' Adjust Window To True Requested Size

  x = MAX&((GetSystemMetrics(%SM_CXSCREEN) - rc.nRight - rc.nLeft) \ 2, 0)
  y = MAX&((GetSystemMetrics(%SM_CYSCREEN) - rc.nBottom - rc.nTop) \ 2, 0)
'
  'Create the dialog...
  hDialog = CreateWindow( zClass, _               'window class name
                          zTitle, _               'window caption
                          dwStyle, _              'window style
                          x, _                    'initial x position
                          y, _                    'initial y position
                          rc.nRight - rc.nLeft, _ 'initial width size
                          rc.nBottom - rc.nTop, _ 'initial height size
                          %HWND_DESKTOP, _        'parent window handle
                          %NULL, _                'window menu handle
                          hInstance, _            'program instance handle
                          byval %NULL )           'creation parameters


  'Show it...
  ShowWindow hDialog, iCmdShow
  UpdateWindow hDialog

  'Process messages...
  while GetMessage( msg, %NULL, 0, 0 )
    TranslateMessage msg
    DispatchMessage msg  
  wend

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function wndproc ( byval hWnd as dword, byval wmsg as long, _
                   byval wParam as long, byval lParam as long ) as long

  static xhTree       as dword  
  static xhDragList   as dword
  static xhCursors    as dword
  static xhDragItem   as dword
  static xuHotSpot    as POINTAPI
  
  local  pNMTV        as NM_TREEVIEW ptr
  local  uIconInfo    as ICONINFO
  local  uPoint       as POINTAPI
  local  uTV_HIT_INFO as TV_HITTESTINFO
  local  uRect        as RECT
  local  hList        as dword
  local  hCursor      as dword
  local  hDropItem    as dword
  local  mIndent      as long                                'per-item indent (pixels)
  local  mLevel       as long                                'item indentation level
  local  mResult      as long
  
  select case wmsg
  
    case %WM_CREATE
    
      'Create the treeview and set up its image list...
      GetClientRect hWnd, uRect
      xhTree = CreateTree( GetModuleHandle( byval %NULL ), hWnd, uRect )
      if xhTree = %NULL then
      
        msgbox "Couldn't create tree !", , $TITLE
        SendMessage hWnd, %WM_CLOSE, 0, 0
      
      else

        'Assign a control ID to the tree...
        SetWindowlong xhTree, %GWL_ID, %ID_TREE

        'Add some dummy data to the tree...
        PopulateTree xhTree
      
        'Create the drag image list, and the drag cursor icon...
        xhCursors = ImageList_Create( 32, 32, %ILC_COLOR4 or %ILC_MASK, 1, 1 )
        hCursor   = LoadCursor( %NULL, byval %IDC_ARROW )
        ImageList_AddIcon xhCursors, hCursor
        GetIconInfo hCursor, uIconInfo

        'Record these in a static variable...
        xuHotSpot.x = uIconInfo.xHotSpot
        xuHotSpot.y = uIconInfo.yHotSpot

        'Clean up unwanted bitmaps...
        DeleteObject uIconInfo.hbmMask
        DeleteObject uIconInfo.hbmColor

        'Patrice...
        IF skInitEngine("Braun.sks", "") THEN
           CALL skSkinWindow(hWnd, "Dock|Undock|Minimize|Maximize|Restore|Close")
           'CALL skSetAnchorCtrl(xhTree, %ANCHOR_CENTER)
           CALL skSetAnchorCtrl(xhTree, %ANCHOR_HEIGHT_WIDTH )
        END IF
      
      end if
      
    case %WM_SIZE
      
      'Have the tree fill the dialog at all times...
      select case wParam

        case %SIZE_MINIMIZED
          'do nothing

'        case %SIZE_RESTORED, %SIZE_MAXIMIZED
'          MoveWindow xhTree, 0, 0, LOWRD(lParam), HIWRD(lParam), %TRUE

      end select
    
    case %WM_NOTIFY
      
      'Respond to tree notifications...
      if wParam = %ID_TREE then
        
        pNMTV = lParam
        select case @pNMTV.hdr.code
          
          case %TVN_ITEMEXPANDED                                'tree item expanded/collapsed

            'Set the item's image according to its new expanded or collapsed status...
            TVW_SetImage xhTree, @pNMTV.ItemNew.hItem, @pNMTV.ItemNew.state and %TVIS_EXPANDED

            'Refresh the whole window...
            InvalidateRect hWnd, byval %NULL, %TRUE
            UpdateWindow hWnd
          
          case %TVN_BEGINDRAG
              
            'We only allow certain tree items to be dragged...
            if TVW_IsDragSource( xhTree, @pNMTV.ItemNew.hItem ) then

              'This call creates a drag image from the tree item, and returns the
              'handle of an image list containing it as item 0. Note that at this
              'stage, the image is based solely on the tree item (it doesn't contain
              'the arrow cursor yet)...
              xhDragList = TreeView_CreateDragImage( xhTree, @pNMTV.ItemNew.hItem )

              'Remove the existing selection...
              TreeView_SelectItem xhTree, %NULL  

              'Record the level this item is at...
              mIndent = TreeView_GetIndent( xhTree )
              mLevel  = TVW_GetLevel( xhTree, @pNMTV.ItemNew.hItem )

              'Get RECT for *entire line* occupied by dragged item. This is relative 
              'to treeview origin (proven)...
              TreeView_GetItemRect xhTree, @pNMTV.ItemNew.hItem, uRect, %FALSE

              '@pNMTV.ptDrag is measured relative to the tree (proven)...
              ImageList_BeginDrag xhDragList, _
                                  0, _
                                  @pNMTV.ptDrag.x - uRect.nLeft - mIndent * mLevel, _
                                  @pNMTV.ptDrag.y - uRect.nTop
              
              'Without this line, the drag image won't contain an arrow...
              ImageList_SetDragCursorImage xhCursors, _
                                      0, _
                                      @pNMTV.ptDrag.x - uRect.nLeft - xuHotSpot.x - mIndent * mLevel, _
                                      @pNMTV.ptDrag.y - uRect.nTop  - xuHotSpot.y
              
              'We're off...
              ImageList_DragEnter xhTree, @pNMTV.ptDrag.x, @pNMTV.ptDrag.y

              'Capture the mouse & hide the normal cursor...
              SetCapture hWnd
              ShowCursor %FALSE
              xhDragItem = @pNMTV.ItemNew.hItem

            end if
            
        end select
        
      end if
    
    case %WM_MOUSEMOVE
    
      'If dragging, move the drag cursor...
      if xhDragList <> %NULL then
        
        'See where we are...
        uPoint.x = LOWRD( lParam )                              'these are relative to <hWnd>
        uPoint.y = HIWRD( lParam )
        
        'Convert to tree co-ordinates...
        MapWindowPoints hWnd, xhTree, byval varptr(uPoint), 1
        
        'Load the hit test structure...
        uTV_HIT_INFO.pt.x = uPoint.x                            'these are relative to <xhTree>
        uTV_HIT_INFO.pt.y = uPoint.y

        'If over a valid drop target, hilight it...
        TreeView_HitTest xhTree, uTV_HIT_INFO
        
        if TVW_IsDropTarget( xhTree, uTV_HIT_INFO.hItem ) and (uTV_HIT_INFO.hItem <> TreeView_GetDropHilight( xhTree )) then
          
          'Clear the drag image...
          ImageList_DragLeave xhTree
          
          'Unlight the old item, hilight the new one...
          TreeView_SelectDropTarget xhTree, %NULL
          TreeView_SelectDropTarget xhTree, uTV_HIT_INFO.hItem
          
          'Show the drag image...
          ImageList_DragEnter xhTree, uPoint.x, uPoint.y
        
        else
          ImageList_DragMove uPoint.x, uPoint.y
        end if
        
        'Check if we need to auto-scroll...
        CheckForAutomaticScroll hWnd, xhTree, uPoint
            
      end if
    
    case %WM_LBUTTONUP
      
      'If dragging, drop the object in its new location...
      if xhDragList <> %NULL then
              
        'Stop the dragging process...
        ImageList_DragLeave xhTree
        ImageList_EndDrag
        ImageList_Destroy xhDragList
        xhDragList = %NULL
        ReleaseCapture
        ShowCursor %TRUE
        
        'We only continue if the mouse is within the tree...
        GetClientRect xhTree, uRect
        GetCursorPos uPoint
        ScreenToClient xhTree, uPoint
        if PtInRect( uRect, uPoint.x, uPoint.y ) then             'inside the tree
        
          'Determine which item was last hilighted...
          hDropItem = TreeView_GetDropHilight( xhTree )

          'Unselect the drop hilight item...
          TreeView_SelectDropTarget xhTree, %NULL

          if hDropItem <> %NULL then

            'drag source = xhDragItem
            'drop target = hDropItem

            if hDropItem <> xhDragItem then

              'Disallow drops onto own children (actually, not possible in this demo)...
              if TVW_IsChildOf( xhTree, hDropItem, xhDragItem ) then
                beep
              else
                'The final aim of the drag & drop operation...
                TVW_MoveItem xhTree, xhDragItem, hDropItem
              end if

            else
              'Disallow drops onto self
              beep
            end if

          end if
        
        end if
        
        'Update the treeview display...
        InvalidateRect xhTree, byval %NULL, %TRUE
      
      end if

    case %WM_TIMER
      if wParam = %IDT_SCROLL then
        
        ImageList_DragShowNoLock %FALSE
          SendMessage xhTree, %WM_VSCROLL, maklng( gmScrollDirection, 0 ), 0
        ImageList_DragShowNoLock %TRUE

        GetCursorPos uPoint
        ScreenToClient xhTree, uPoint
        CheckForAutomaticScroll hWnd, xhTree, uPoint
        
      end if
    
    case %WM_CLOSE
      hList = TreeView_GetImageList( xhTree, %TVSIL_NORMAL )
      if hList     then ImageList_Destroy hList
      if xhCursors then ImageList_Destroy xhCursors           'suspect unnecessary
    
    case %WM_DESTROY    
      PostQuitMessage 0
      mResult = 0
      goto Bye_wndproc
  
  end select

  'If we get to here, the message wasn't handled, or needs completing by the default handler...
  mResult = DefWindowProc( hWnd, wmsg, wParam, lParam )

Bye_wndproc:
  function = mResult
  exit function

end function
